package com.muyu.marketing.team.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.muyu.common.core.domain.Result;
import com.muyu.common.core.utils.StringUtils;
import com.muyu.common.core.web.page.TableDataInfo;

import com.muyu.marketing.team.mapper.ActivityTeamInfoMapper;
import com.muyu.marketing.team.service.ActivityTeamInfoService;
import com.muyu.marketing.team.service.ActivityTeamOpenInfoService;
import com.muyu.marketing.team.service.ActivityTeamProductSkuInfoService;

import com.muyu.markteing.domain.ActivityTeamInfo;
import com.muyu.markteing.domain.model.ActivityTeamInfoListModel;
import com.muyu.markteing.domain.model.ActivityTeamInfoListQueryModel;
import com.muyu.markteing.domain.model.TeamProductDiscountPriceModel;
import com.muyu.markteing.domain.model.TeamProductStockModel;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.List;

@Service
public class ActivityTeamInfoServiceImpl extends ServiceImpl<ActivityTeamInfoMapper, ActivityTeamInfo> implements ActivityTeamInfoService {

    @Autowired(required=true)
    private ActivityTeamOpenInfoService activityTeamOpenInfoService;

    @Autowired(required=true)
    private ActivityTeamProductSkuInfoService activityTeamProductSkuInfoService;


    @Override
    public TableDataInfo<ActivityTeamInfoListQueryModel> query(ActivityTeamInfoListModel activityTeamInfoListModel) {



        LambdaQueryWrapper<ActivityTeamInfo> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.like(StringUtils.isNotEmpty(activityTeamInfoListModel.getKeyWord()),ActivityTeamInfo::getName,activityTeamInfoListModel.getKeyWord());
        queryWrapper.like(StringUtils.isNotEmpty(activityTeamInfoListModel.getStatus()),ActivityTeamInfo::getStatus,activityTeamInfoListModel.getStatus());


        Page<ActivityTeamInfo> activityTeamInfoPage1 = this.page(activityTeamInfoListModel.buildPage(), queryWrapper);
        List<ActivityTeamInfo> activityTeamInfoList = activityTeamInfoPage1.getRecords();
        List<ActivityTeamInfoListQueryModel> activityTeamInfoListQueryModels = activityTeamInfoList.stream()
                .map(activityTeamInfo ->  ActivityTeamInfoListQueryModel.infoBuild(
                        activityTeamInfo, (activityTeamInfoListQueryModelBuilder)-> {
                    TeamProductStockModel teamProductStockModel = activityTeamProductSkuInfoService.getStock(activityTeamInfo.getId());
                    TeamProductDiscountPriceModel discountPrice = activityTeamProductSkuInfoService.getDiscountPrice(activityTeamInfo.getId());
                    Long teamOpenNumber = activityTeamOpenInfoService.getTeamOpenNumberByTeamId(activityTeamInfo.getId());
                    Long teamInTypeNumber = activityTeamOpenInfoService.getTeamInTypeNumberByTeamId(activityTeamInfo.getId());
                   return activityTeamInfoListQueryModelBuilder
                            .openTeamNumber(teamOpenNumber)
                            .addTeamNumber(teamInTypeNumber)
                            .attendNumber(teamOpenNumber + teamInTypeNumber)
                            .teamPrice(discountPrice.getTeamPrice())
                            .productPrice(discountPrice.getProductPrice())
                            .teamStock(teamProductStockModel.getTeamStock())
                            .remainStock(teamProductStockModel.getRemainStock())
                            .build();
                })).toList();

        TableDataInfo<ActivityTeamInfoListQueryModel> tableDataInfo = new TableDataInfo<>();
        tableDataInfo.setTotal(activityTeamInfoPage1.getTotal());
        tableDataInfo.setRows(activityTeamInfoListQueryModels);
        return tableDataInfo;
    }



}


