package com.muyu.marketing.team.service.impl;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.muyu.common.core.exception.ServiceException;

import com.muyu.marketing.team.mapper.ActivityTeamProductSkuInfoMapper;
import com.muyu.marketing.team.service.ActivityTeamProductSkuInfoService;
import com.muyu.markteing.domain.ActivityTeamProductSkuInfo;
import com.muyu.markteing.domain.model.TeamProductDiscountPriceModel;
import com.muyu.markteing.domain.model.TeamProductStockModel;
import com.muyu.product.cache.ProjectSkuCache;
import com.muyu.product.domain.ProjectSkuInfo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Optional;

@Service
public class ActivityTeamProductSkuInfoServiceImpl  extends ServiceImpl<ActivityTeamProductSkuInfoMapper, ActivityTeamProductSkuInfo> implements ActivityTeamProductSkuInfoService {

   @Autowired
   private ProjectSkuCache projectSkuCache;
    @Override
    public TeamProductDiscountPriceModel getDiscountPrice(Long teamId) {
        List<ActivityTeamProductSkuInfo> teamProductSkuInfoList = this.getActivityTeamProductSkuInfoByTeamId(teamId);

        Optional<TeamProductDiscountPriceModel> discountPriceModelOptional = teamProductSkuInfoList.stream().map(activityTeamProductSkuInfo -> {
            ProjectSkuInfo projectSkuInfo = projectSkuCache.get(activityTeamProductSkuInfo.getProductId(), activityTeamProductSkuInfo.getProductSku());
            return TeamProductDiscountPriceModel.of(projectSkuInfo.getPrice(), activityTeamProductSkuInfo.getTeamPrice());
        }).min((o1, o2) -> Double.valueOf(o1.getDiscount() * 100 - o2.getDiscount() * 100).intValue());


        if (discountPriceModelOptional.isEmpty()){
            throw new ServiceException("团购活动下没有商品绑定");
        }

        return discountPriceModelOptional.get();
    }

    @Override
    public TeamProductStockModel getStock(Long teamId) {
        List<ActivityTeamProductSkuInfo> teamProductSkuInfoList = this.getActivityTeamProductSkuInfoByTeamId(teamId);

        return TeamProductStockModel.builder()
                .teamStock(teamProductSkuInfoList.stream().map(ActivityTeamProductSkuInfo::getTeamStock).reduce(0L,Long::sum))
                .remainStock(teamProductSkuInfoList.stream().map(ActivityTeamProductSkuInfo::getRemainStock).reduce(0L,Long::sum))
                .build();
    }
}
